﻿using System;
using System.Activities;
using System.Activities.Core.Presentation;
using System.Activities.Presentation;
using System.Activities.Presentation.Toolbox;
using System.Activities.Presentation.Validation;
using System.Activities.Presentation.View;
using System.Activities.Statements;
using System.Activities.XamlIntegration;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.ServiceModel.Activities;
using System.ServiceModel.Activities.Presentation.Factories;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Win32;

namespace CustomWorkflowDesigner
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private WorkflowDesigner _workflowDesigner;
        private string _fileName;

        public MainWindow()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(MainWindow_Loaded);
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            new DesignerMetadata().Register();

            var toolbox = new ToolboxControl();
            var cat = new ToolboxCategory("Standard Activities");
            var assemblies = new List<Assembly>();
            assemblies.Add(typeof(Send).Assembly);
            assemblies.Add(typeof(Delay).Assembly);
            assemblies.Add(typeof(ReceiveAndSendReplyFactory).Assembly);

            var query = from asm in assemblies
                        from type in asm.GetTypes()
                        where type.IsPublic &&
                        !type.IsNested &&
                        !type.IsAbstract &&
                        !type.ContainsGenericParameters &&
                        (typeof(Activity).IsAssignableFrom(type) ||
                        typeof(IActivityTemplateFactory).IsAssignableFrom(type))
                        orderby type.Name
                        select new ToolboxItemWrapper(type);

            query.ToList().ForEach(ti => cat.Add(ti));
            toolbox.Categories.Add(cat);
            Grid.SetColumn(toolbox, 0);
            Grid.SetRow(toolbox, 1);
            LayoutGrid.Children.Add(toolbox);
        }

        private void OpenButton_Click(object sender, RoutedEventArgs e)
        {
            var dlg = new OpenFileDialog();
            if (dlg.ShowDialog() == true)
            {
                _fileName = dlg.FileName;
                ClearWorkflowDesigner();
                LoadWorkflowDesigner();
            }
        }

        private void ClearWorkflowDesigner()
        {
            if (_workflowDesigner != null)
            {
                LayoutGrid.Children.Remove(_workflowDesigner.View);
                LayoutGrid.Children.Remove(_workflowDesigner.PropertyInspectorView);
                _workflowDesigner = null;
            }
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            if (_workflowDesigner != null && !string.IsNullOrEmpty(_fileName))
            {
                _workflowDesigner.Save(_fileName);
            }
        }

        private void RunButton_Click(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrEmpty(_fileName))
            {
                try
                {
                    var writer = new StringWriter();
                    var workflow = ActivityXamlServices.Load(_fileName);
                    var wa = new WorkflowApplication(workflow);
                    wa.Extensions.Add(writer);
                    wa.Completed = WorkflowCompleted;
                    wa.OnUnhandledException = WorkflowUnhandledException;
                    wa.Run();

                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void WorkflowCompleted(WorkflowApplicationCompletedEventArgs e)
        {
            var writer = e.GetInstanceExtensions<StringWriter>().First();
            MessageBox.Show(writer.ToString(), "Workflow Completed");
        }
        private UnhandledExceptionAction WorkflowUnhandledException(WorkflowApplicationUnhandledExceptionEventArgs e)
        {
            MessageBox.Show(e.UnhandledException.Message, "Workflow Unhandled Exception");
            return UnhandledExceptionAction.Terminate;
        }

        private void LoadWorkflowDesigner()
        {
            _workflowDesigner = new WorkflowDesigner();
            _workflowDesigner.Load(_fileName);

            var view = _workflowDesigner.View;
            Grid.SetColumn(view, 1);
            Grid.SetRow(view, 1);
            LayoutGrid.Children.Add(view);

            var propInspector = _workflowDesigner.PropertyInspectorView;
            Grid.SetColumn(propInspector, 2);
            Grid.SetRow(propInspector, 1);
            LayoutGrid.Children.Add(propInspector);

            var validationErrorService = new ValidationErrorService(WorkflowErrors.Items);
            _workflowDesigner.Context.Services.Publish<IValidationErrorService>(validationErrorService);

            _workflowDesigner.Context.Items.Subscribe<Selection>(SelectionChanged);
        }

        private void SelectionChanged(Selection selection)
        {
            var modelItem = selection.PrimarySelection;
            var sb = new StringBuilder();

            while (modelItem != null)
            {
                var displayName = modelItem.Properties["DisplayName"];

                if (displayName != null)
                {
                    if (sb.Length > 0)
                        sb.Insert(0, " - ");
                    sb.Insert(0, displayName.ComputedValue);
                }

                modelItem = modelItem.Parent;
            }

            CurrentActivityName.Text = sb.ToString();
        }
    }
}
